from drivers import BaseDriver, setup_readwrite
from Task import Task, MajorFault, MinorFault
import re
import ladder_operands


INSTRUCTIONS = {
	r'XIC':ladder_operands.XIC,
	r'XIO':ladder_operands.XIO,
	r'OTE':ladder_operands.OTE
}
RUNG_END = r';'
BRANCH_START = r'BST'
BRANCH_END = r'BND'
BRANCH_NEXT = r'NXB'


class Ladder(Task):
	def custom_cfg(self, cfg):
		if 'logic' in cfg:
			self.logic = cfg['logic']

	def custom_read(self, tag):
		if tag == 'logic':
			return {'logic': self.logic}
		
	def custom_write(self, tag, value):
		if tag == 'edit':
			print 'edited'
			self.logic = value

	def task_logic(self, read, write):
		preparse(self.logic, read, write)


def preparse(text, read, write):
	rungs = text.split(RUNG_END)
	for rung in rungs:
		pieces = rung.split(' ')[::-1]
		parse(pieces, True, read, write)

def parse(pieces, power_state, read, write):
	piece = ''
	power_state_initial = power_state
	power_state_branch_result = False

	while pieces:
		piece = pieces.pop()
		if piece in INSTRUCTIONS:
			inst = INSTRUCTIONS[piece]
			ops = pieces[- inst.operand_count:]
			pieces = pieces[:- inst.operand_count]
			
			logic = inst(ops)
			logic.input = power_state
			power_state = logic.logic(read, write)

		elif piece ==  BRANCH_START:
			pieces, power_state = parse(pieces, power_state, read, write)
		elif piece ==  BRANCH_NEXT:
			power_state_branch_result = power_state_branch_result or power_state
			power_state = power_state_initial
		elif piece ==  BRANCH_END:
			power_state_branch_result = power_state_branch_result or power_state
			return pieces, power_state_branch_result

